// $Header: /home/dvermeir/cvsroot/dvnet/dvnet/inetaddress.C,v 1.7 2003/10/31 14:36:17 dvermeir Exp $
// release: $Name:  $

#include <config.h>

#ifdef	HAVE_UNISTD_H
#include <unistd.h>	// gethostname()
#endif

#ifdef	HAVE_SYS_SOCKET_H
#include <sys/socket.h>	// AF_INET
#endif

#ifdef	HAVE_ARPA_INET_H
#include <arpa/inet.h>	// inet_ntoa()
#endif

#ifdef	HAVE_NETDB_H
#include <netdb.h>	// struct hostent
#endif

#ifdef HAVE_NETINET_IN_H
#include <netinet/in.h> // for *BSD
#endif

#include <errno.h>
#include <iostream>

#include "inetaddress.h"

using namespace std;
namespace Dv {
namespace Net {
InetAddress::InetAddress(const char* host,unsigned long addr, const char *da):
		host_(host), address_(addr), dot_address_(da) {
}

bool
InetAddress::operator==(const InetAddress& a) const {
// Back to basics: 2 inet addresses are the same if they have the same address,
// irrespective of funny names.
return (address_==a.address_);
}

Dv::Util::ref<InetAddress>
InetAddress::by_name(const string& shost) {
const char*       host(shost.c_str());
static const int  MAXBUF	= 4048;
static char	  buf[MAXBUF];
int		  error;
struct hostent	  he;
struct hostent*   hr(0);

#ifdef HAVE_FUNC_GETHOSTBYNAME_R_6
if (gethostbyname_r(host,&he,buf,MAXBUF,&hr,&error)!=0)
  hr = 0;

#elif HAVE_FUNC_GETHOSTBYNAME_R_5

hr = gethostbyname_r(host,&he,buf,MAXBUF,&error);

#elif HAVE_FUNC_GETHOSTBYNAME_R_3

{
struct hostent_data hostent_data_buf;
if (gethostbyname_r(host,&he,&hostent_data_buf)==0)
  hr = &he;
else
  hr = 0;
}
#elif HAVE_GETHOSTBYNAME
#warning "using gethostbyname instead of gethostbyname_r makes this lib not mt-safe"
hr = gethostbyname(host);  
#else
#error "no gethostbyname_r or gethostbyname"
#endif

// We can ignore error as (error==0) implies (hr==0).
if (!hr) 
  return Dv::Util::ref<InetAddress>(0);	// not found
if (hr->h_addrtype!=AF_INET) 
  return Dv::Util::ref<InetAddress>(0);
  
in_addr	addr = *reinterpret_cast<struct in_addr*>(*(hr->h_addr_list));
// Addr is in network order, and so is inet_ntoa(addr).
return Dv::Util::ref<InetAddress>(new InetAddress(host, addr.s_addr, inet_ntoa(addr)));
}

Dv::Util::ref<InetAddress>
InetAddress::by_address(unsigned long addr) {
static const int  MAXBUF = 4048;
static char	  buf[MAXBUF];
int	          error;
struct	hostent	  he;
struct	hostent*  hr;

#ifdef HAVE_FUNC_GETHOSTBYADDR_R_7
hr = gethostbyaddr_r(reinterpret_cast<const char *>(&addr),sizeof(addr),AF_INET,
                     &he,buf,MAXBUF,&error);
#elif HAVE_FUNC_GETHOSTBYADDR_R_8
gethostbyaddr_r(reinterpret_cast<const char *>(&addr),sizeof(addr),AF_INET,
                &he,buf,MAXBUF,&hr,&error);
#elif HAVE_GETHOSTBYADDR
#warning "using gethostbyaddr instead of gethostbyaddr_r makes this lib not mt-safe"
hr = gethostbyaddr(reinterpret_cast<const char *>(&addr), sizeof(addr),
                   AF_INET);
#else
#error "no gethostbynaddr_r or gethostbynaddr"
#endif

if (!hr)
  return Dv::Util::ref<InetAddress>(0);
if (hr->h_addrtype!=AF_INET) 
  return Dv::Util::ref<InetAddress>(0);

in_addr	ia;
ia.s_addr = addr;
return Dv::Util::ref<InetAddress>(new InetAddress(hr->h_name, ia.s_addr, inet_ntoa(ia)));
}

string
InetAddress::local_host() {
static const int MAXBUF = 256;
static char buf[MAXBUF];
if (gethostname(buf, sizeof(buf)) == -1)
  strcpy(buf, "localhost");
return string(buf);
}

}}
