// $Id: message.C,v 1.7 2003/07/05 12:04:43 dvermeir Exp $
#include <config.h>
#include "message.h"

using namespace std;

namespace Dv {
namespace Net {
static unsigned char hexchars[] = "0123456789ABCDEF";

static inline int
hexdigit(char c) { // assert(isxdigit(c))
if (isdigit(c)) 
  return c - '0';
else if (isupper(c))
  return c - 'A' + 10;
else // must be lower case letter
  return c - 'a' +10;
}

const char*
SimpleMessage::data(const char* p, size_t size) { 
if (owner_)
  delete data_;
data_ = p;
size_ = size;
return p;
}

char*
SimpleMessage::data(size_t size) { 
data(new char[size],size);
owner_ = true;
return const_cast<char*>(data_);
}

ostream&
SimpleMessage::send(ostream& os) const {
size_t enc_size(size_*2);
char* enc_data = new char[enc_size];
char* q(enc_data);
size_t n(0);
for (const char* p = data_; n<size_; ++n, ++p) {
  *q++ = hexchars[static_cast<unsigned char>(*p)>>4];
  *q++ = hexchars[static_cast<unsigned char>(*p)&15];
  }
os << enc_size << endl;
os.write(enc_data, enc_size);
os.rdbuf()->STREAMBUF_PUBSYNC();
delete enc_data;
return os;
}

istream&
SimpleMessage::receive(istream& is) throw (runtime_error) {
static const size_t max_len_size(10); // maximal number of bytes
// needed for an ascii encoding of the size of the message
char enc_size_buf[max_len_size];
// Istream::getline will fail if the line has more than max_len_size-1
// characters, see [Josuttis p. 608]. Note also that istream::getline
// will read at most max_len_size-1 characters, so the 2nd parameter is OK
// (it does not have to be max_len_size-1 to leave room for the terminating
// 0).
if (! is.getline(enc_size_buf, max_len_size) )
  throw std::runtime_error("Dv::Util::SimpleMessage::receive: cannot read enc_size");
size_t enc_size(atoi(enc_size_buf));
if (enc_size %2 )
  throw std::runtime_error("Dv::Util::SimpleMessage::receive: message size not even");
// is.ignore(); // \n following size
char* enc_data = new char[enc_size];
if (! is.read(enc_data, enc_size) )
  throw runtime_error("SimpleMessage::receive: could not read enc_data");

const char* p = enc_data;

char* q = data(enc_size/2);
for (size_t n = 0; n<size(); ++n, ++q) {
  *q = 16 * hexdigit(*p) + hexdigit(*(p+1));
  p += 2;
  }
delete enc_data;
return is;
}

}}
