// $Id: test-continue-after-timeout.C,v 1.2 2003/10/31 14:36:17 dvermeir Exp $

#include <unistd.h> // for fork(), sleep()
#include <sys/types.h> // wait()
#include <sys/wait.h> // wait()
#include <fstream>
#include <string>

#include <dvnet/socket.h>
#include <dvnet/serversocket.h>

using namespace Dv;
using namespace Net;

const std::string ERROR("error: ");
const std::string NAME("test-continue-after-timeout: ");
const std::string CLIENT(NAME + "client: ");
const std::string SERVER(NAME + "server: ");

static const int server_delay(900); // millisec
static const int client_delay(1); //sec, make sure server_delay * 2 > client_delay * 1000

int
error(const std::string& mesg, int retcode=1) {
std::cerr << mesg << std::endl;
return retcode;
}

std::ostream&
socket_status(const std::string& prefix, Socket& client, std::ostream& os) {
return os 
     << prefix << "strerror = " << client.strerror() << std::endl
     << prefix << "error    = " << client.error() << std::endl
     << prefix << "timedout = " << client.timedout() << std::endl
     << prefix << "fd       = " << client.sfd() << std::endl
     ;
}

int // return 0 iff ok
echo_client(const std::string& host,int port) {

std::cerr << CLIENT << "connecting to " << host << ":" << port << std::endl;

Socket	client(host, port);

if (!client) 
  return error(CLIENT + ERROR + "connection failed: " + client.strerror());

client << 'a' << std::endl;

std::cerr << CLIENT << "sleeping " << client_delay << " secs" << std::endl;
sleep(client_delay);

socket_status(CLIENT, client, std::cerr);

std::string prompt;
if ( getline(client,prompt) )
  std::cerr << CLIENT << "received prompt = " << prompt << std::endl;
else 
  socket_status(CLIENT, client, std::cerr);

std::cerr << CLIENT << "sending reply" << std::endl;
if ( client << "better late than never" << std::endl )
  std::cerr << CLIENT << "sent reply ok" << std::endl;
else 
  socket_status(CLIENT, client, std::cerr);


return (client.error()); // should be ok
}

int
main(int argc,char* argv[]) {

Dv::Util::ref<InetAddress> ia(InetAddress::by_name("localhost"));

if (!ia) 
  return error(ERROR + "cannot determine InetAddress of local host");

const int port(9999);

std::cerr << NAME << "server will be on " << ia->host() << ":" << port << std::endl;

bool	ok(false);

// The parent will be the server, the child the client.

int child = fork();

if (child<0) 
  return error(ERROR + "fork() failed");
  
if (child==0) {
  sleep(3);
  std::cerr << CLIENT << "start" << std::endl;
  return echo_client(ia->host(),port);
  }

try {
  ServerSocket	server(port);
  std::cerr << SERVER << "start" << std::endl;

  Dv::Util::ref<Socket>	client(server.accept(server_delay)); 

  std::cerr << SERVER << "connection from " 
       << client->host() << "." << client->port() << std::endl;

  unsigned int n=0;
  char c;
  while (*client >> c)
    ++n;

  socket_status(SERVER, *client, std::cerr) << SERVER << "n bytes  = " << n << std::endl;

  std::cerr << SERVER << "resetting socket" << std::endl;

  // client->rdbuf()->debug(&std::cerr);
  client->timedout(false);
  socket_status(SERVER, *client, std::cerr);

  std::cerr << SERVER << "sending prompt" << std::endl;
  if ( *client << "hurry up" << std::endl )
    std::cerr << SERVER << "prompt send ok" << std::endl;
  else {
    std::cerr << SERVER << "prompt send error" << std::endl;
    socket_status(SERVER, *client, std::cerr);
    return error(SERVER, 1);
    }

  std::string reply;

  if (! std::getline(*client, reply) ) {
    std::cerr << SERVER << "error receiving reply" << std::endl;
    socket_status(SERVER, *client, std::cerr);
    return error(SERVER, 1);
    }
  else
    std::cerr << SERVER << "received reply: " << reply << std::endl;

  }
catch (std::exception& e) {
  return error(SERVER + e.what(),2);
  }

int client_status;
std::cerr << SERVER << "waiting for client to die" << std::endl;
wait(&client_status);
std::cerr << SERVER << "client died, status = " << client_status << std::endl;

if (WIFEXITED(client_status)) 
   if (WEXITSTATUS(client_status)==0)
     return ok;
   else
     return 1;
else
  return 1;
}
