// $Id: test-echocs.C,v 1.4 2003/10/31 14:36:17 dvermeir Exp $

#include <unistd.h> // for fork(), sleep()
#include <sys/types.h> // wait()
#include <sys/wait.h> // wait()
#include <fstream>
#include <string>

#include <dvnet/socket.h>
#include <dvnet/serversocket.h>

using namespace std;
using namespace Dv;
using namespace Net;

int // return 0 iff ok
echo_client(const string& host,int port) {
cerr << "Client connecting to " << host << ":" << port << endl;
Socket	client(host,port);

if (!client) {
  cerr	<< "connection failed: " << client.strerror() << endl;
  return 1;
  }

ifstream ifs("/etc/passwd");
if (!ifs) {
  cerr << "Cannot open /etc/passwd" << endl;
  return 1;
  }

string line;
string ifsline;
while (getline(ifs,ifsline)) {
  client << ifsline << endl;
  getline(client,line);
  if (line!=ifsline) {
    cerr << "Client expected \"" << ifsline << "\", got \"" << line << "\"" << endl;
    return 1;
    }
  }
cout << "Client exit:" << client.strerror() << endl;
return client.error();
}

int
main(int,char**) {
const char* const usage = "techo";

// Dv::Util::ref<InetAddress> ia(InetAddress::by_name(InetAddress::local_host()));
Dv::Util::ref<InetAddress> ia(InetAddress::by_name("localhost"));
if (!ia) {
  cerr << "Cannot determine InetAddress of local host" << endl;
  return 1;
  }
int port(9999);

cerr << "Server will be on " << ia->host() << ":" << port << endl;


// The parent will be the server, the child the client.

int child = fork();

if (child<0) {
  cerr << "fork() failed" << endl;
  return 1;
  }
  
if (child==0) {
  sleep(3);
  cerr << "Starting client.." << endl;
  return echo_client(ia->host(),port);
  }

try {
  ServerSocket	server(port);
  cerr << "Starting server.." << endl;

  Dv::Util::ref<Socket>	client(server.accept());

  cout << "Connection from " << client->host() << ":" << client->port() << endl;

  string line;
  while (getline(*client,line))
    *client << line << endl;
  cout << "client: " << client->strerror() << endl;
  }
catch (exception& e) {
  cerr << e.what() << endl;
  return 2;
  }

int client_status;
int child_exit_status; 
wait(&child_exit_status);
std::cerr << "main: child_exit_status = " << child_exit_status << std::endl;

if (WIFEXITED(child_exit_status)) {
   client_status = WEXITSTATUS(child_exit_status);
   std::cerr << "main: client client_status = " << child_exit_status << std::endl;
   }
else {
  if (WIFSIGNALED(child_exit_status)) {
    std::cerr << "main: WIFSIGNALED: signal = " << WTERMSIG(child_exit_status) << std::endl;
    client_status = 1;
    }
  else {
    std::cerr << "server: WIFEXITED returns strange stuff" << std::endl;
    client_status = 2;
    }
  }
return client_status;
}
