// $Id: test-message-echo.C,v 1.1 2003/05/10 11:36:29 dvermeir Exp $
#include <unistd.h> // for fork(), sleep()
#include <sys/types.h> // wait()
#include <sys/wait.h> // wait()
#include <dvutil/systemerror.h>
#include <dvnet/socket.h>
#include <dvnet/serversocket.h>

#include "message.h"

struct A {
  int i;
  int j;
  A(int ii = 0, int jj = 0): i(ii), j(jj) {}
  bool operator==(const A& a) { return (i == a.i) && (j == a.j); }
  bool operator!=(const A& a) { return ! (*this == a); }
};

std::istream&
operator>>(std::istream& is, A& a) {
return is >> a.i >> a.j;
}

std::ostream&
operator<<(std::ostream& os, const A& a) {
return os << a.i << " " << a.j;
}

///////////////////////////////////////////////////////////////////////////////////////////////
// Send an XML file and receive it back from the server, return true
// if all went well.
///////////////////////////////////////////////////////////////////////////////////////////////
bool
echo_xml(const std::string& filenm, const std::string& host, int port) {

std::cerr << "client: connecting to " << host << ":" << port << std::endl;
try {
  Dv::Net::Socket socket(host, port);

  if (!socket) 
    throw std::runtime_error(socket.strerror() + ": connection failed");

  A a_out(3,5);

  // send a message
  Dv::Net::Message<A> m_out(a_out);
  m_out.send(socket);
  Dv::Net::Message<A> m_in;
  m_in.receive(socket);

  return (a_out == m_in);
  // std::string sent_string;
  // sent_document.save_to_string(sent_string);
  // std::cerr << "client: document sent: " << sent_string << std::endl;

  //Dv::Xml::send(sent_document, socket);

  return true;
  // receive a document
  // xml::document received_document;
  // std::string received_string;

  // Dv::Xml::receive(received_document, socket);
  //received_document.save_to_string(received_string);
  //std::cerr << "client: document received: " << received_string << std::endl;

  // compare both documents
  // return sent_string == received_string;
  }
catch (std::exception& e) {
  std::cerr << "client: " << e.what() << std::endl;
  return false;
  }
}

///////////////////////////////////////////////////////////////////////////////////////////////
int
main(int argc,char* argv[]) {

try {
  Dv::Util::ref<Dv::Net::InetAddress> ia(Dv::Net::InetAddress::by_name("localhost"));

  if (!ia) 
    throw std::runtime_error("Cannot determine InetAddress of local host");

  int port(9999);

  std::cerr << "main: server on " << ia->host() << ":" << port << std::endl;


  // The parent will be the server, the child the client.
  int child = fork();

  if (child<0)
    throw Dv::Util::SystemError();
  
  ////////////////////////////////////////////////////////////////////////////////////////////////
  // Client fork
  if (child==0) {
    std::cerr << "client: sleeping .." << std::endl;
    sleep(5);
    std::cerr << "client: starting .." << std::endl;
    return ( echo_xml("test.xml", ia->host(), port) ? 0 : 1 );
    }
  //
  ////////////////////////////////////////////////////////////////////////////////////////////////

  Dv::Net::ServerSocket server(port);
  std::cerr << "server: start.." << std::endl;
  
  Dv::Util::ref<Dv::Net::Socket> client( server.accept() );
  
  std::cerr << "server: accepted connection from "
            << client->host() << ":" << client->port() << std::endl;

  Dv::Net::Message<A> m;
  m.receive(*client);
  m.send(*client);
  // xml::document document;
  // Dv::Xml::receive(document, *client);
  // std::cerr << "server: received document: " << document << std::endl;
  // Dv::Xml::send(document, *client);

  int client_status;
  wait(&client_status);

  if (WIFEXITED(client_status)) 
     if (WEXITSTATUS(client_status)==0)
       return 0;
     else
       return 1;
  else
    return 1;
  }
catch (std::exception& e) {
  std::cerr << "server: " << e.what() << std::endl;
  return 2;
  }
}
