// $Id: test-timeout-accept.C,v 1.3 2004/12/22 11:10:21 dvermeir Exp $

#include <unistd.h> // for fork(), sleep()
#include <sys/types.h> // wait()
#include <sys/wait.h> // wait()
#include <string.h> // strerror()
#include <fstream>
#include <string>

#include <dvnet/socket.h>
#include <dvnet/serversocket.h>

using namespace std;
using namespace Dv;
using namespace Net;

int // return 0 iff ok
echo_client(const string& host,int port) {
std::cerr << "client: connecting to " << host << ":" << port << endl;
Socket	client(host,port);
client.debug(&std::cout);

if (!client) {
  cerr	<< "client: connection failed: " << client.strerror() << endl;
  return 1;
  }

ifstream ifs("/etc/passwd");
if (!ifs) {
  cerr << "client: cannot open /etc/passwd" << endl;
  return 1;
  }

size_t n_lines(0);
string line;
string ifsline;
while (getline(ifs,ifsline)) {
  ++n_lines;
  client << ifsline << endl;
  getline(client,line);
  if (line!=ifsline) {
    std::cerr << "client: expected \"" << ifsline << "\", got \"" << line << "\"" << endl;
    return 1;
    }
  }
std::cerr << "client: lines read = " << n_lines << endl;
std::cerr << "client: exit:" << client.strerror() << endl;
return client.error();
}

int
main(int,char**) {

// ref<InetAddress> ia(InetAddress::by_name(InetAddress::local_host()));
Dv::Util::ref<InetAddress> ia(InetAddress::by_name("localhost"));
if (!ia) {
  cerr << "server: cannot determine InetAddress of local host" << endl;
  return 1;
  }
int port(9999);

std::cerr << "server: port = " << ia->host() << ":" << port << endl;


// The parent will be the server, the child the client.
try {

  ServerSocket	server(port);

  int syserr;
  if ( server.connection(1000, &syserr) )
    throw std::runtime_error("server.connection should fail but didn't");
  std::cerr << "main/server: expected connection() error: " 
            << strerror(syserr) << std::endl;

  int child = fork();

  if (child<0) {
    std::cerr << "main: fork() failed" << endl;
    return 1;
    }
    
  if (child==0) {
    sleep(3);
    std::cerr << "child: starting client.." << endl;
    return echo_client(ia->host(),port);
    }

  std::cerr << "server: waiting for connection" << endl;

  while (! server.connection(1000) )
    std::cerr << "server: waiting for connection" << std::endl;
  
  std::cerr << "server: connection, start accept" << std::endl;
  Dv::Util::ref<Socket> client(server.accept());

  client->debug(&std::cout);
  std::cerr << "server: connection from " << client->host() << ":" << client->port() << endl;

  string line;
  while (getline(*client,line))
    *client << line << endl;
  std::cerr << "server: client status: " << client->strerror() << endl;
  }
catch (exception& e) {
  std::cerr << "main: exception: " << e.what() << endl;
  return 2;
  }

int client_status;
int child_exit_status; 
wait(&child_exit_status);
std::cerr << "main: child_exit_status = " << child_exit_status << std::endl;

if (WIFEXITED(child_exit_status)) {
   client_status = WEXITSTATUS(child_exit_status);
   std::cerr << "main: client client_status = " << child_exit_status << std::endl;
   }
else {
  if (WIFSIGNALED(child_exit_status)) {
    std::cerr << "main: WIFSIGNALED: signal = " << WTERMSIG(child_exit_status) << std::endl;
    client_status = 1;
    }
  else {
    std::cerr << "server: WIFEXITED returns strange stuff" << std::endl;
    client_status = 2;
    }
  }
return client_status;
}
