// $Id: test-timeout-client-read.C,v 1.3 2003/10/31 14:36:17 dvermeir Exp $

#include <unistd.h> // for fork(), sleep()
#include <sys/types.h> // wait()
#include <sys/wait.h> // wait()
#include <fstream>
#include <string>

#include <dvnet/socket.h>
#include <dvnet/serversocket.h>

using namespace std;
using namespace Dv;
using namespace Net;

const string ERROR("error: ");
const string NAME("test-timeout-client-read: ");
const string CLIENT(NAME + "client: ");
const string SERVER(NAME + "server: ");

// delays in seconds.
static const int client_delay(1);
static const int server_delay(client_delay*5);

int
error(const string& mesg, int retcode=1) {
cerr << mesg << endl;
return retcode;
}

int // return 0 iff ok
echo_client(const string& host,int port) {

cerr << CLIENT << "connecting to " << host << ":" << port << endl
     << CLIENT << "delay = " << client_delay << "sec" << endl;

bool ok(false);

{
Socket	client(host, port, 100, client_delay*1000); // delay of 1 sec

if (!client) 
  return error(CLIENT + ERROR + "connection failed: " + client.strerror());

client.rdbuf()->debug(&cerr);
unsigned int n=0;
char c;
while (client>>c)
  ++n;

cerr << CLIENT << "strerror = " << client.strerror() << endl
     << CLIENT << "error    = " << client.error() << endl
     << CLIENT << "timedout = " << client.timedout() << endl
     << CLIENT << "rdbuf.strerror    = " << client.rdbuf()->strerror() << endl
     << CLIENT << "rdbuf.error    = " << client.rdbuf()->error() << endl
     << CLIENT << "rdbuf.timedout = " << client.rdbuf()->timedout() << endl
     << CLIENT << "n bytes  = " << n << endl
     ;

ok = (client.timedout()?true:false);
}
cerr << CLIENT << "exiting" << endl;;
return (ok?0:1);
}

int
main(int argc,char* argv[]) {

Dv::Util::ref<InetAddress> ia(InetAddress::by_name("localhost"));

if (!ia) 
  return error(ERROR + "cannot determine InetAddress of local host");

const int port(9999);

cerr << NAME << "server will be on " << ia->host() << ":" << port << endl;


// The parent will be the server, the child the client.

int child = fork();

if (child<0) 
  return error(ERROR + "fork() failed");
  
if (child==0) {
  sleep(3);
  cerr << CLIENT << "start" << endl;
  return echo_client(ia->host(),port);
  }

try {
  ServerSocket	server(port);
  cerr << SERVER << "start" << endl;

  Dv::Util::ref<Socket>	client(server.accept()); 

  cerr << SERVER << "connection from " 
       << client->host() << "." << client->port() << endl;

  cerr << SERVER << "server: sleeping " << server_delay << " sec" <<  endl; 
  
  *client << 'a' << endl;

  sleep(server_delay);

  cerr << SERVER << "strerror = " << client->strerror() << endl
       << SERVER << "error    = " << client->error() << endl
       << SERVER << "timedout = " << client->timedout() << endl
       ;

  }
catch (exception& e) {
  return error(SERVER + e.what(),2);
  }

int client_status;
cerr << SERVER << "waiting for client to die" << endl;
wait(&client_status);
cerr << SERVER << "client died" << endl;

if (WIFEXITED(client_status)) 
   if (WEXITSTATUS(client_status)==0)
     return 0;
   else
     return 1;
else
  return 1;
}
