// $Id: test-udelayclient.C,v 1.1 2002/10/18 18:15:37 dvermeir Exp $

#include <unistd.h> // for fork(), sleep()
#include <sys/types.h> // wait()
#include <sys/wait.h> // wait()
#include <fstream>
#include <string>
#include <dvutil/date.h>
#include <dvnet/usocket.h>
#include <dvnet/userversocket.h>

using namespace std;
using namespace Dv;
using namespace Net;

/* The server accepts a connection but then does not read its
   input for a number of secs which is longer than the client
   is prepared to wait. Thus the client should fail upon
   reading data back. The server will fail with EPIPE because
   it eventually receives the data, but when it writes them
   back, the process on the other size has already gone away
   (``Broken pipe'').

*/
int // return 0 iff ok
echo_client(const string& servername) {

cerr << Util::Date().str() << " client: connecting to " << servername << endl;

usocket	client(servername,3000);
// usocket	client(servername,0);

if (!client) {
  cerr << Util::Date().str() << " client: connection failed: "
       << client.strerror() << endl;
  return 1;
  }
else 
  cerr << Util::Date().str() << " client: connection ok: "
       << client.strerror() << endl;
  
ifstream ifs("/etc/passwd");
if (!ifs) {
  cerr << Util::Date().str() << "client: cannot open /etc/passwd" << endl;
  return 1;
  }

string line;
string ifsline;
int	n(0);
while (getline(ifs,ifsline)) {
  client << ifsline << endl;
  if (!client) {
    cerr << Util::Date().str() << " client: write failed: "
         << client.strerror() << endl;
    return 0;
    }
  getline(client,line);
  if (!client) {
    cerr << Util::Date().str() << " client: read failed: "
         << client.strerror() << endl;
    return 0;
    }
  // since client does not reply, we expect
  if (line!=ifsline) {
    cerr << Util::Date().str() << " client: expected \"" 
         << ifsline << "\", got \"" << line << "\"" << endl;
    return 0;
    }
  ++n;
  }
cerr << Util::Date().str() << " client: read " << n << " lines" << endl;
cerr << Util::Date().str() << " client: exit: " << client.strerror() << endl;
return client.error();
}

int
main(int argc,char* argv[]) {
// const char* const usage = "tuechocs";
const string servername("/tmp/tudelay-client.sock");


int child = fork();

if (child<0) {
  cerr << "fork() failed" << endl;
  return 1;
  }
  
if (child==0) {
  sleep(1);
  cerr << Util::Date().str() << " client: starting" << endl;
  return echo_client(servername);
  }

try {
  cerr << Util::Date().str() << " server: starting" << endl;
  userversocket	server(servername,0);
  if (!server) {
    cerr << Util::Date().str()
         << "server: creation failed: " << server.strerror() << endl;
    return 2;
    }
  cerr << Util::Date().str() << " server accepting" << endl;
  Dv::Util::ref<usocket> client(server.accept(5000));

  cerr << Util::Date().str() << " server: accepted connection " << endl;

  if (!*client) {
    cerr << Util::Date().str() << " server: accept error: "
         << client->strerror() << endl; 
    return 3;
    }
  cerr << Util::Date().str() << " server: accept ok" << endl;
  cerr << Util::Date().str() << " server: sleeping" << endl; sleep(6);
  cerr << Util::Date().str() << " server: wakeup" << endl;
  string line;
  client->rdbuf()->debug(&cerr);
  while (getline(*client,line)) {
    cerr << Util::Date().str() << " server: write" << endl;
    if (!*client) 
      cerr << Util::Date().str() 
           << "server: I/O error: " << client->strerror() << endl;
    else
      *client << line << endl;
    }
  cerr << Util::Date().str() 
       << "server: error: " << client->strerror() << endl;
  return 0;
  }
catch (exception& e) {
  cerr << "server: exception: " << e.what() << endl;
  return 2;
  }

cerr << Util::Date().str() << "server: done" << endl;
int client_status;
wait(&client_status);

if (WIFEXITED(client_status)) 
   if (WEXITSTATUS(client_status)==0)
     return 0;
   else
     return 1;
else
  return 1;
}
