#ifndef	DV_NET_USERVERSOCKET_H
#define	DV_NET_USERVERSOCKET_H

// $Id: userversocket.h,v 1.4 2002/10/18 18:15:37 dvermeir Exp $
#include	<stdexcept>
#include	<dvutil/ref.h>
#include	<dvnet/usocket.h>

/*! \file
    This file defines the Dv::Net::usocket (client) class.
*/
namespace Dv {
namespace Net {

//! Class representing a server listening on a unix socket.
class userversocket {
public:
  //! Constructor.
  /*! \param filename associated with newly created unix socket
      \param delay in millisecs that will be associated with connections to the server
      \param nconnections queue size of unix socket
  */
  explicit userversocket(const std::string& filename, time_t delay=0, int nconnections=5);
  //! Destructor.
  ~userversocket();

  //! Accept a client connection. 
  /*! \param delay in millisecs before accept() will time out
      \param bufsz that will be associated with client socket

      The success can be tested by testing the status of
      the returned usocket.
      \code
      ref<usocket> client(server.accept());

      if (*client) { 
        // all is well
        }
      \endcode
  */
  Util::ref<usocket>  accept(time_t delay=0,size_t bufsz=1024) {
  	return usocket::accept(*this,delay,bufsz);
	}
  //! Filename of underlying unix socket.
  std::string	filename() const { return fn_; }
  //! Max delay in millisecs that accept will wait before failing.
  time_t	delay() const { return delay_; }
  //! File descriptor of unix socket.
  int		fd() const { return fd_; }
  //! Return whether server is ok.
  bool		status() const { return errno_ == 0; }
  //! Return whether server is ok.
  operator bool() const { return status(); }
  //! Return error code, 0 == ok.
  int		error() const { return errno_; }
  //! Return status string.
  std::string	strerror() const;
private:
  userversocket(const userversocket&);
  userversocket& operator=(const userversocket&);

  std::string	fn_;
  int		fd_;
  int   	errno_;
  time_t	delay_;
};

}}

#endif
