// $Id: usocket.C,v 1.6 2002/12/26 10:31:47 dvermeir Exp $


#include <iostream>
#include <string>
#include <stdio.h> // sprintf
#include <errno.h> // errno
#include <unistd.h> // getpid(), unlink(), fcntl()
#include <fcntl.h> // fcntl()
#include <dvutil/file.h> // Directory::tempfile()

#include "usocksetup.h"
#include "usockstreambuf.h"
#include "usocket.h"
#include "userversocket.h"


using namespace std;

namespace Dv {
namespace Net {

usocket::usocket(const string& servname, time_t delay, const string& filename):
  iostream(new usockstreambuf(1024,1024,delay)), servername_(servname),
  temp_(filename.size()==0) {
string fn(filename);
delay = 0;
if (temp_)
  try {
    Dv::Util::Directory	dir("/tmp");
    fn = dir.tempfile("usocket.");
    }
  catch (exception& e) {
    clear(ios::failbit);
    return;
    }

rdbuf()->filename(fn);

int syserrno;
int sfd(usockstreambuf::bind(fn,syserrno));

if (sfd<0) {
  rdbuf()->error(syserrno);
  rdbuf()->fd(-1);
  clear(ios::failbit);
  return;
  }

// Set fd of usockstreambuf, so destructor will close it.

rdbuf()->fd(sfd);

/********************************************
  The following code should -- but doesn't quite work. The idea is
  to have a working timeout (of delay millisecs) for connect().
  According to several sources (and implementations such as
  CommonC++) one should:
  - make the socket non-blocking
  - call connect() 
  - if connect() returns -1 and errno == EINPROGRESS, then
    - call select() [with timeout] to wait for ouput-would-not-block
      on the socket
    - is select fails, the whole thing timed out
    - else, ``manually'' check the status of the connection using
      getsockopt
********************************************/

/*
// Make socket non-blocking, if we have a max delay.
if (delay>0) 
  if (fcntl(sfd, F_SETFL, FNDELAY) < 0) {
    rdbuf()->error(errno);
    rdbuf()->fd(-1);
    clear(ios::failbit);
    return;
    }
*/
/********************************************
  Try to connect.
********************************************/

struct sockaddr_un server_addr;
socklen_t          server_len;

setup_unsocket(servername(), server_addr, server_len);

int connect_status;
do {
  connect_status = ::connect(sfd, reinterpret_cast<sockaddr*>(&server_addr), server_len);
  }
while ( (connect_status < 0) && (errno == EINTR) );

if (connect_status == 0)
  return; // all is well

syserrno = errno;

/********************************************
  Immediate connection failed.

  If the socket is non-blocking or the error is not EINPROGRESS,
  something definitely went wrong (this is probably redundant
  since syserrno==EINPROGRESS can only occur for a non-blocking
  socket).

********************************************/

if ((delay==0)||(syserrno!=EINPROGRESS)) {
  rdbuf()->error(syserrno);
  rdbuf()->fd(-1);
  clear(ios::failbit);
  return;
  }

/********************************************
  Connect is ``in progress'': wait until it is
  available for output, then test its status using
  getsocktopt().
********************************************/

int ok_for_write = Util::fdstreambuf::fdwait(rdbuf()->fd(),delay,syserrno,true);

if (ok_for_write<=0) { // definitely no connection, probably timed out
  rdbuf()->error(syserrno);
  rdbuf()->fd(-1);
  clear(ios::failbit);
  return;
  }

// Check status of socket.

socklen_t optlen(sizeof(syserrno));
syserrno = 1;

if (getsockopt(rdbuf()->fd(),SOL_SOCKET,SO_ERROR,&syserrno,&optlen)<0) {
  syserrno = errno;
  rdbuf()->error(syserrno);
  rdbuf()->fd(-1);
  clear(ios::failbit);
  return;
  }

if (syserrno!=0) {
  rdbuf()->error(syserrno);
  rdbuf()->fd(-1);
  clear(ios::failbit);
  return;
  }

// Should be ok.
}


usocket::~usocket() {
if (temp_) 
  try {
    Dv::Util::File	f(rdbuf()->filename());
    f.rm();
    }
  catch (exception& e) {
    // ignore
    }
delete rdbuf(); 
}

usocket::usocket(userversocket& server, time_t delay, size_t bufsz):
  iostream(new usockstreambuf(bufsz,bufsz,server.delay())), temp_(false) {

struct sockaddr_un client_addr;
socklen_t          client_len = sizeof(client_addr);
int                syserrno;

int ok = Util::fdstreambuf::fdwait(server.fd(),delay,syserrno);
if (ok>0) {
  int sfd;
  do {
    sfd = ::accept(server.fd(),
  		  reinterpret_cast<sockaddr*>(&client_addr),&client_len);
    }
  while (sfd < 0 && errno == EINTR);
  if (sfd>=0) {
    rdbuf()->fd(sfd);
    return;
    }
  syserrno = errno;
  }
rdbuf()->error(syserrno);
clear(ios::failbit);
return;
}

Util::ref<usocket>
usocket::accept(userversocket& server, time_t delay, size_t bufsz) {
return Util::ref<usocket>(new usocket(server,delay,bufsz));
}

}}
