#ifndef	DV_NET_USOCKET_H
#define	DV_NET_USOCKET_H

// $Id: usocket.h,v 1.7 2002/10/18 18:15:37 dvermeir Exp $
#include	<iostream>
#include	<dvutil/ref.h>
#include	<dvnet/usockstreambuf.h>

/*! \file
    This file defines the Dv::Net::Usocket class.
*/
namespace Dv {
namespace Net {
class userversocket;
//! Unix socket iostream specialization.
class usocket: public std::iostream {
public:

  //! Open client socket on filename connected to servername.
  /*! If filename has size 0, a temporary file will be created in /tmp
  */
  explicit usocket(const std::string& servername, time_t millisecs=0, 
  	const std::string& filename="");

  /** Construct a connection accepted by a server.
      \param srv server doing the accept
      \param delay (in millisecs) that we are prepared to wait, 0 means forever.
      \param bufsize of resulting socket

      Upon failure or timeout, the usocket is still created, but its
      failbit will be set, which can be checked as follows.

      \code
      usocket connection(server,10,bufsize);
      if (connection) {
        }
      \endcode

      The delay (for I/O operations) of the new socket, is inherited from
      the userversocket.
  */
  usocket(userversocket& srv, time_t delay, size_t bufsize);

  //! Construct a connection accepted by a server.
  static Util::ref<usocket> accept(userversocket& srv, time_t delay, size_t bufsize); 

  //! Destructor.
  ~usocket();

  //! Return filename of server.
  const std::string& servername() const { return servername_; }

  //! Return associated usockstreambuf object.
  usockstreambuf* rdbuf(void) const { 
    return static_cast<usockstreambuf*>(std::istream::rdbuf()); 
    }

  //! Return last system error.
  int error() const { return rdbuf()->error(); }
  //! Return last system error string.
  std::string strerror() const { return rdbuf()->strerror(); }

private:

  usocket(const usocket&);
  usocket& operator=(const usocket&);

  std::string servername_;
  bool temp_;
};
}}
#endif
