#ifndef	USOCKSTREAMBUF_H
#define	USOCKSTREAMBUF_H
// $Id: usockstreambuf.h,v 1.3 2002/10/18 18:15:37 dvermeir Exp $
#include	<dvutil/fdstreambuf.h>
#include	<string>
#include	<stdexcept>

/** \file
 *  The Dv::Net::usockstreambuf class specializes
 *  Dv::Util::fdstreambuf for use by Dv::Net::usockstream.
 */
namespace Dv {
namespace Net {
/**
 * Dv::Util::fdstreambuf specialization for unix sockets.
 */
class usockstreambuf: public Dv::Util::fdstreambuf { 
/**
 * Usocket is friend to allow access to Dv::Util::fdstreambuf::error(int)
 */
friend class usocket;
public:
  /**
   * Constructor.
   *
   * \param in_buf_size size (in bytes) of input buffer.
   * \param out_buf_size size (in bytes) of output buffer.
   * \param delay (in milliseconds) if an I/O operation does not
   *   complete in delay milliseconds, it will time out.
   *
   * \see Dv::Util::fdstreambuf::timedout(), Dv::Util::fdstreambuf
   */
  explicit usockstreambuf(size_t in_buf_size=1024, 
  			  size_t out_buf_size=1024,
			  time_t delay=0);
  /**
   * Destructor.
   */
  virtual ~usockstreambuf();

  /** 
   * Create socket fd and bind to filename, return <0 if error.
   *
   * \param filename to which socket will be bound
   * \param err will receive error status, i.e. last system errno
   * \return 0 iff ok, -1 else. If -1 is returned, err will be set
   *   to the last system errno.
   */
  static int    	bind(const std::string& filename, int& err);
  /**
   * \return filename of this unix socket.
   */
  const std::string&		filename() const { return fn_; }

private:
  /**
   * Set filename associated with this unix socket.
   * \param fn filename to associate with this socket.
   */
  void			filename(const std::string& fn) { fn_ = fn; }
  /**
   * Filename associated with this unix socket.
   */
  std::string		fn_;
};

}}

#endif
